/*-
* renameFiles is a simple tool to rename many files at once
* Copyright (C) 2011 David Hobach
* tripleh [at] hackingthe [dot] net
* 
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
-*/

#include "AbstractQObjectWriter.h"

#include <QFile>
#include <QByteArray>
#include <QMetaProperty>

bool AbstractQObjectWriter::writeToFile(const QString& path, const QObject* obj, bool writeChildren)
{
QFile file(path);

QByteArray arr = this->toBytes(obj,writeChildren);
if (arr.size() == 0) return false;

if (!file.open(QIODevice::WriteOnly)) return false; //QIODevice::Text isn't really needed even when writing text (possibly the case here) - see doc
int written = file.write(arr);
file.close();

if (written != arr.size()) return false;
return true;
}

QStringList AbstractQObjectWriter::propertyNames(const QObject* obj) const
{
const QMetaObject* meta = obj->metaObject();
QStringList retval;

QList<QByteArray> dynamicProps = obj->dynamicPropertyNames(); //get dynamic property names

foreach (QByteArray qba, dynamicProps) retval << QString(qba);

for (int i=0; i < meta->propertyCount(); ++i) retval << meta->property(i).name(); //get static property names

return retval;
}
