/*-
* renameFiles is a simple tool to rename many files at once
* Copyright (C) 2011 David Hobach
* tripleh [at] hackingthe [dot] net
* 
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
-*/

/** a simple interface for processing command line arguments
* An object which provides a simple interface to the command line arguments of a program. The methods @ref getSwitch(QString) and @ref getSwitchArg(QString)  allow you to process and remove the switches and switched  arguments in the command line, so that the remaining entries in the stringlist can be processed as a  uniform list. 
* It also happens to be derived from QStringList, so you can use any of those methods on this object too.
* This helps you avoid entirely using the C arrays in your application.  
* @author S. Alan Ezust sae@mcs.suffolk.edu
* @since qt 3.2.1
* @see http://cartan.cas.suffolk.edu/oopdocbook/opensource/argumentlist.html
*/

#ifndef ARGUMENTLIST_H
#define ARGUMENTLIST_H

#include <QStringList>


class ArgumentList : public QStringList 
{

public:

/** Constructor
* retrieve argument list from the qApp methods. Only works if a @ref QCoreApplication(argc, argv) or QApplication(argc, argv) was already created.
*/
ArgumentList();

/**
* @param argc number of arguments
* @param argv an array of command line arguments, as supplied to main().
* @see argsToStringList()
*/
ArgumentList(int argc, char* argv[]) {argsToStringlist(argc, argv);}

ArgumentList(const QStringList& argumentList) : QStringList(argumentList) {}

/** finds and removes a switch from the string list, if it exists.
* @param option/switch to search for
* @return true if the switch was found
*/
bool getSwitch(const QString& option);

/** finds/removes a switch and its accompanying argument from the string list, if the switch is found. Does nothing if the switch is not found.
* @param option/switch to search for
* @param the return value if option is not found in the stringlist
* @return the argument following the switch/option or defaultRetVal if the option is not found
*/
QString getSwitchArg(const QString& option, const QString& defaultRetVal = QString());


private:
/** (Re)loads argument lists into this object. This function is private because it is part of the implementation of the class and not intended to be used by client code.
*/
void argsToStringlist(int argc, char* argv[]);
	
}; //end class

#endif

