/*-
* renameFiles is a simple tool to rename many files at once
* Copyright (C) 2011 David Hobach
* tripleh [at] hackingthe [dot] net
* 
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
-*/

/** simple argument handling class similar to Argumentlist.h, but using stdlib instead of Qt 
 * @author David Hobach
 * @version 0.5 / 20.06.2011
 */

#ifndef ARGUMENTLIST_STD_H
#define ARGUMENTLIST_STD_H

#include <vector>
#include <string>
#include <ostream>

class ArgumentList_std : public std::vector<std::string> 
{

public:

/**
* @param argc number of arguments
* @param argv an array of command line arguments, as supplied to main().
* @see argsToStringList()
*/
ArgumentList_std(int argc, char* argv[]) {argsToStringlist(argc, argv);}

/** copy ctor
 */
ArgumentList_std(const ArgumentList_std& argumentList);

/** finds and removes a switch from the string list, if it exists.
* @param option/switch to search for
* @return true if the switch was found
*/
bool getSwitch(const std::string& option);

/** finds/removes a switch and its accompanying argument from the string list, if the switch is found. Does nothing if the switch is not found.
* @param option/switch to search for
* @param the return value if option is not found in the stringlist
* @return the argument following the switch/option or defaultRetVal if the option is not found
*/
std::string getSwitchArg(const std::string& option, const std::string& defaultRetVal = std::string());

/** convert the object to a string
 */
std::string toString() const
{
std::string ret;
	for (size_t i = 0; i < this->size()-1; ++i)
	{
	ret.append((*this)[i]);
	ret.append(" ");
	}
if (this->size() > 0) ret.append((*this)[this->size()-1]);
return ret;
}

/** write all arguments to the stream in their original form
 */
friend std::ostream& operator<<(std::ostream& out, const ArgumentList_std& args)
{
out << args.toString();
return out;
}

private:
/** (Re)loads argument lists into this object. This function is private because it is part of the implementation of the class and not intended to be used by client code.
*/
void argsToStringlist(int argc, char* argv[]);

}; //end class

#endif

