/*-
* renameFiles is a simple tool to rename many files at once
* Copyright (C) 2011 David Hobach
* tripleh [at] hackingthe [dot] net
* 
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
-*/

#include <QMetaObject>
#include <QMetaProperty>

#include "BasicQObjectWriter.h"
#include "Streams.h"

BasicQObjectWriter::BasicQObjectWriter(bool useNumbers, bool showTypes, QVariantHelper* qvh) : m_useNumbers(useNumbers), m_showTypes(showTypes)
{
	if (qvh == 0)
	{
	m_allocated = true;
	m_qvh = new QVariantHelper();
	}
	else
	{
	m_allocated = false;
	m_qvh = qvh;
	}
}

QByteArray BasicQObjectWriter::toBytes(const QObject* obj, bool writeChildren)
{
QString ret = this->toString(obj,writeChildren);
if (ret == QString()) return QByteArray();
return ret.toLocal8Bit();
}

QString BasicQObjectWriter::toString(const QObject* obj, bool writeChildren)
{
QStringList ret;
if (!this->toString(ret,obj,writeChildren,0)) return QString();
return ret.join("\n");
}

BasicQObjectWriter::~BasicQObjectWriter()
{
	if ((m_allocated) && (m_qvh != 0))
	{
	delete m_qvh;
	m_qvh = 0;
	}
}

bool BasicQObjectWriter::toString(QStringList& result, const QObject* obj, bool writeChildren, int indentLevel, QString lastNumber, int curNumber) const
{
QString indentspace, value, type;
indentspace.fill('\t',indentLevel);
QString completeNumber("");
	
	if (m_useNumbers)
	{
	if (lastNumber.length() > 0) completeNumber = QString("%1.%2").arg(lastNumber).arg(curNumber);
	else completeNumber = QString("%1").arg(curNumber);
	}

QString spacer;
spacer.fill(' ',completeNumber.length());
QString className = obj->metaObject()->className();
QString objectName = obj->objectName();
objectName.replace("\n","\n" + indentspace + spacer + " "); //so that the indention remains ok even if the value contains newlines

result << QString("\n%1%2 class: %3").arg(indentspace).arg(completeNumber).arg(className); //\n for the sake of beauty
//result << QString("%1%2 properties:").arg(indentspace).arg(spacer);
	if (m_showTypes) type = " (QString)";
	else type = "";
result << QString("%1%2 objectName%3: %4").arg(indentspace).arg(spacer).arg(type).arg(objectName); 
type = "";

QStringList propnames = this->propertyNames(obj); //inherited
	foreach (QString propName, propnames)
	{
	if (propName == "objectName") continue; //already done, otherwise we seem to have some problems with the object name
	QVariant qv = obj->property(propName.toLocal8Bit());
	
		if (propName == "className") 
		{
		className = qv.toString();
		continue;
		}
	
	const QMetaObject* meta = obj->metaObject();
	int idx = meta->indexOfProperty(propName.toLocal8Bit());
	QMetaProperty mprop = meta->property(idx);
	
	bool ok;
	value = m_qvh->toString(qv,mprop,ok); //note: QString() is returned by toString(), if the QVariant can't be converted
	value.replace("\n","\n" + indentspace + spacer + " "); //so that the indention remains ok even if the value contains newlines
	if (!ok) return false;
		if (m_showTypes)
		{
		type = QString(" (%1)").arg(qv.typeName());
		}
	result << QString("%1%2 %3%4: %5").arg(indentspace).arg(spacer).arg(propName).arg(type).arg(value);
	}

	if (writeChildren) //work recursively, if flag set
	{
	QList<QObject*> childlist = obj->findChildren<QObject*>(QString());
	QObject* child;
		for (int i = 0; i < childlist.size(); ++i)
		{
		child = childlist[i];
			if (child->parent() != obj)
			{
			//qDebug() << "This is not my child!!";
			continue;
			}
			if ((child != 0) && (!this->toString(result,child,writeChildren,indentLevel+1,completeNumber,i+1))) return false;
		}
	}
return true; //if we get here, the conversion QVariant --> QString of the value was possible at least in this run of the recursion
}
