/*-
* renameFiles is a simple tool to rename many files at once
* Copyright (C) 2011 David Hobach
* tripleh [at] hackingthe [dot] net
* 
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
-*/

/** simple implementation of the interface AbstractQObjectWriter for human readable output using indention to symbolize the QObject hierarchy and optionally nunmbering the QObjects
* could be used for debugging purposes or wherever readability is important
* corresponding reader: not implemented (yet?)
* @author David Hobach
* @version 0.3 / 21.05.2010
*/

#ifndef BASICQOBJECTWRITER_H
#define BASICQOBJECTWRITER_H

#include "AbstractQObjectWriter.h"
#include "QVariantHelper.h"

class BasicQObjectWriter : public AbstractQObjectWriter
{

public:

/** ctor
* @param whether or not to use numbers and indention to represent the QObject hierarchy or just use indention (optional, default is to just use indention)
* @param whether or not to show the property types in the output
* @param pointer to a QVariantHelper to use for handling the various types that may occur in the QObjects to write with this writer (if set to NULL, the basic QVariantHelper implementation is used)
*/
BasicQObjectWriter(bool useNumbers = false, bool showTypes = true, QVariantHelper* qvh = 0); //ctor useNumbers param so that the interface rules are not violated

/** get a QByteArray basic human readable representation of the specified QObject
* @param pointer to QObject to represent as QByteArray (basically an ASCII QString here)
* @param whether or not to inspect and recursively write child objects of the object to write (optional)
* @return QByteArray basic human readable representation of the specified QObject; it has size 0, if the operation wasn't successful; NOTE: since the return value is a String, the function just calls this.toString().toAscii()
*/
virtual QByteArray toBytes(const QObject* obj, bool writeChildren = true);

/** same as toBytes, but returns the basic human readable representation of the QObject as QString
* @param pointer to the QObject to represent in XML
* @param whether or not to inspect and recursively write child objects of the object to write (optional)
* @return QString representation of the QObject; == QString(), if the operation wasn't successful (if some properties had types, that couldn't be converted from QVariant)
*/
virtual QString toString(const QObject* obj, bool writeChildren = true);

//implementation of base class is sufficient
//bool writeToFile(const QString& path, const QObject* obj, bool writeChildren = true);

//setters
void showTypes(bool showTypes) {m_showTypes = showTypes;}
void useNumbers(bool useNumbers) {m_useNumbers = useNumbers;}

//getters
bool usingNumbers() {return m_useNumbers;}
bool showingTypes() {return m_showTypes;}

/** dtor
*/
virtual ~BasicQObjectWriter();

protected:

/** same as the public version, just with specification of the indentlevel for the implementation of recursion
* @param QStringList to use for the return value: QString representation of the QObject and possibly all of its children
* @param pointer to the QObject to represent
* @param whether or not to inspect and recursively write child objects of the object to write (optional)
* @param level of indention for recursion (uses tabs)
* @param QString used with the last number
* @param current number
* @return whether all values of the properties could be correctly converted by the QVariantHelper = true
*/
virtual bool toString(QStringList& result, const QObject* obj, bool writeChildren, int indentLevel, QString lastNumber = QString(), int curNumber = 1) const;

private:
bool m_useNumbers;
bool m_showTypes;
bool m_allocated; //whether m_qvh was allocated by this class or not
QVariantHelper* m_qvh;

}; //end class

#endif
