/*-
* renameFiles is a simple tool to rename many files at once
* Copyright (C) 2011 David Hobach
* tripleh [at] hackingthe [dot] net
* 
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
-*/

#include "QMimeFileInfo.h"

#include <QtGlobal>
#include <QProcess>
#include <QStringList>
#include <QRegExp>

/** execute a binary with the given parameters
 * @param binary to execute
 * @param parameters to be passed to the binary
 * @return output of the binary
 */
QString execProcess(const QString& name, const QStringList& params)
{
QProcess proc;
proc.start(name,params,QIODevice::ReadOnly);
        if (!proc.waitForStarted())
        {   
        proc.kill();
        return ""; 
        }   
        if (!proc.waitForFinished())
        {   
        proc.kill();
        return ""; 
        }   
QString resultStr(proc.readAll());
resultStr.remove(QRegExp("\\s+$")); //remove trailing whitespace
return resultStr;
}

QString QMimeFileInfo::getMimeType(bool includeEncoding) const
{
#ifdef Q_OS_LINUX
	if (!this->exists()) return "";

	QFileInfo filebin("/usr/bin/file");
	if (!filebin.isExecutable()) return "";

	//we simply use the external file utility
	//-b : don't include the file name in the output
	//-i : mime type and encoding
	//--mime-type: mime type only
	QStringList params;
		if (includeEncoding) params << "-bi";
		else
		{
		params << "-b";
		params << "--mime-type";
		}
	params << this->canonicalFilePath();
	return execProcess("file",params);
#else
	return "";
#endif
}

bool QMimeFileInfo::isBinary()
{
if (readFirstBytesToCache() <= 0) return false;
bool ret = false;
char* data = m_cache->data();
	if ((data[0] == 'M') && (data[1] == 'Z')) ret = true; //DOS header, supported by ALL windows binaries
	else if ((data[0] == '\x7f') && (data[1] == 'E') && (data[2] == 'L') && (data[3] == 'F')) ret = true; //posix elf binaries (includes *.so files), a.out
	else if ((data[0] == '!') && (data[1] == '<') && (data[2] == 'a') && (data[3] == 'r') && (data[4] == 'c') && (data[5] == 'h') && (data[6] == '>')) ret = true; //ar format (static libaries in linux and windows, *.a, *.lib) //http://en.wikipedia.org/wiki/Ar_%28Unix%29#File_format_details

if (!m_useCache) deleteCache();
return ret;
}

bool QMimeFileInfo::isText()
{
if (readFirstBytesToCache() < 0) return false; //0 is acceptable here, file might be empty
bool cache_save;
bool ret = false;
cache_save = m_useCache;
m_useCache = true; //we want to use the cache at least once
	if (!isBinary())
	{
	if (this->fileName().endsWith(".txt",Qt::CaseInsensitive)) ret = true;
	else if ((qstrlen(m_cache->data()) == ((unsigned int)m_cache->size())) && (!this->fileName().endsWith(".pdf"))) ret = true; //no \0's --> can be completely converted to a string (this seems to work really well except for pdf files)
	//else if (this->getMimeType().contains("text")) ret = true;
	}
m_useCache = cache_save;
if (!m_useCache) deleteCache();
return ret;
}

void QMimeFileInfo::setMimeCaching(bool useMimeCaching)
{
m_useCache = useMimeCaching;
if (!useMimeCaching) deleteCache();
}

int QMimeFileInfo::readFirstBytesToCache(int numBytes)
{
	if (m_cache)
	{
		if (m_cache->size() >= numBytes) return m_cache->size(); //cache hit
		else
		{
		//the cache doesn't contain enough bytes
		deleteCache();
		}
	}

//case: we need to read the bytes from the file
m_cache = new QByteArray();
QFile file(this->absoluteFilePath());
if (!file.open(QIODevice::ReadOnly)) return -1;
*m_cache = file.read(numBytes); //copy -_-
file.close();
	if (file.error() != QFile::NoError)
	{
	deleteCache();
	return -1;
	}
return m_cache->size();
}

