/*-
* renameFiles is a simple tool to rename many files at once
* Copyright (C) 2011 David Hobach
* tripleh [at] hackingthe [dot] net
* 
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
-*/

#include <QObject>

#include "QObjectFactory.h"
#include "Streams.h"

using std::auto_ptr;

namespace factories
{

//init statics
QHash<QString,pfnConstructor> QObjectFactory::s_map = QHash<QString,pfnConstructor>();
auto_ptr<QObjectFactory> QObjectFactory::s_instance(0);

QObjectFactory* QObjectFactory::instance(QObject* parent) //static must not be mentioned here!
{
	if (!s_instance.get()) {s_instance.reset(new QObjectFactory(parent));}
	else
	{
	if (parent != 0) s_instance.get()->setParent(parent);
	}
return s_instance.get();
}

QObject* QObjectFactory::build(const QString& className, QObject* parent)
{
pfnConstructor pointer = s_map.value(className,0); //return 0, if it doesn't exist
if (pointer == 0) return 0;
QObject* ret = (*pointer)(); //beware: if this function should return something else, bad things might happen...
	if (ret != 0)
	{
	if (parent != 0) {ret->setParent(parent);}
	else ret->setParent(this);
	}
return ret;
}

QObjectFactory::QObjectFactory(QObject* parent) : QObject(parent) {;}

QObjectFactory::~QObjectFactory()
{
if ((s_instance.get() == 0) || (s_instance.get()->parent() != 0)) s_instance.release(); //then it will be destroyed by its parent instead of the auto_ptr
//cout << "dtor called." << endl; //for debugging
}

} //end namespace
