/*-
* renameFiles is a simple tool to rename many files at once
* Copyright (C) 2011 David Hobach
* tripleh [at] hackingthe [dot] net
* 
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
-*/

/** simple namespace for definition of a reasonable == operator for QObjects
* if this comparison is meant to work correctly, your QObjects should implement MetaObject setters and getters (properties)
* if you need more definitions of ==, implement a Comparator mechanism similar to that one of Java
*/

#ifndef QOBJ_SUPPORT
#define QOBJ_SUPPORT
 
#include <QObject>
#include <QVariant>
#include <QString>
#include <QTextStream>

namespace QObjectSupport
{

/** if any property of the two QObjects differs, false is returned
* This version of operator== is used primarily from assertEquals. It should be made more robust - currently it only checks whether the properties are the same - it should also do other kinds of checks.
*/
bool operator==(const QObject& o1, const QObject& o2);

QTextStream& operator<<(QTextStream& os, const QVariant & qv) ;
//QTextStream& operator<<(QTextStream& os, const QObject& qo) ;
// QTextStream& operator<<(QTextStream& os, const QString& qs) ;

} //end namespace

#endif
