/*-
* renameFiles is a simple tool to rename many files at once
* Copyright (C) 2011 David Hobach
* tripleh [at] hackingthe [dot] net
* 
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
-*/

#include "QObjectTree.h"
#include "QObjectSupport.h"
//#include "Streams.h"

//using namespace QObjectSupport;

QObjectTree::QObjectTree(QObject* parent) : QObject(parent)
{
//cout << "QObjectTree dtor called." << endl;
this->current = this;
this->removed = new QObject(parent);
}

QObjectTree::~QObjectTree()
{
//the QObject destructor should destroy subobjects (is called in reverse order = last)
removed->setParent(0);
delete removed;
}

QObject* const QObjectTree::getRoot()
{
return this;
}

QObject* const QObjectTree::getCurrent()
{
return this->current;
}

const QList<QObject*>& QObjectTree::getCurrentSubNodes()
{
return this->current->children();
}

void QObjectTree::addSubNodeToCurrent(QObject* node)
{
node->setParent(this->current); //children list is updated by Qt
emit treeChanged(this->current);
}

QObject* const QObjectTree::remove(QObject* node)
{
//we should NEVER call a destructor directly and we don't know if the user allocated the node on the stack or the heap --> don't call delete either
//just mark it as cleanable by adding it to removed
//the only thing one could do, is to provide a bool variable to indicate a wanted delete - but then again, the user could also clean the heap himself
if (!findPointer(node)) return 0;
if (node == this->getRoot()) return 0;
QObject* parent = node->parent();
node->setParent(removed); //note, that maybe ret != &node
emit treeChanged(parent);
return node;
}

bool QObjectTree::moveToPointer(QObject* node)
{
if (this->current == node) return true; //for performance
if (!findPointer(node)) return false;
this->current = node;
return true;
}

void QObjectTree::reset()
{
this->current = this->getRoot();
}

bool QObjectTree::findPointer(const QObject* node)
{
if (node == this->getRoot()) return true; //special case, since root = this in our case
return findPointerR(this->getRoot(),node);
}

bool QObjectTree::findPointerR(const QObject* start, const QObject* find)
{
QList<QObject*> children = start->children();
	for (int i = 0; i < children.size(); ++i)
	{
	if (children[i] == find) return true; //pointer comparison
	if (findPointerR(children[i],find)) return true;
	}
return false;
}
