/*-
* renameFiles is a simple tool to rename many files at once
* Copyright (C) 2011 David Hobach
* tripleh [at] hackingthe [dot] net
* 
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
-*/

/** class to make usage of QVariant easier, especially its conversion to QString
* NOTE: if you just need conversion to and from basic types, use QVariant.value<basic type T>() or convert() and canConvert()
* to support user defined/custom types (own classes) with QVariant, simply derive this class and implement them (also don't forget to register the type using the Q_DECLARE_METATYPE macro)
* used by: probably at least some implementations of AbstractQObjectWriter
* @author David Hobach
* @version 0.3 / 11.04.2010
* @since Qt 4.6
*/

#ifndef QVARIANTHELPER_H
#define QVARIANTHELPER_H

#include <QVariant>
#include <QString>
#include <QMetaProperty>

class QVariantHelper
{

public:

/** convert the QVariant of the specified type (specified in the QMetaProperty) to a String
* this basic implementation supports all types QVariant::toString() supports plus enumerations and the class QTime
* @param QVariant to convert
* @param meta property specifying the type of the QVariant
* @param OUTPUT variable to indicate whether the operation was successful or not
* @return String representation of that QVariant, if ok == true; for parsing it could be good to use Qt::escape() on that return value
*/
virtual QString toString(const QVariant& var, const QMetaProperty& meta, bool& ok) const;   

/** dtor
*/
virtual ~QVariantHelper() {;}

}; //end class


#endif
