/*-
* renameFiles is a simple tool to rename many files at once
* Copyright (C) 2011 David Hobach
* tripleh [at] hackingthe [dot] net
* 
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
-*/

#include <QtGlobal> //Q_OS macro
#include <QFile>
#include <QIODevice>

#include "TextFileReadWriteHelper.h"
#include "Streams.h"

void TextFileReadWriteHelper::setQTextStreamCodec(QTextStream* stream)
{
	if (m_codec == "")
	{
	//unfortunately Qt doesn't seem to do this itself, so we do it
	//NOTE: Latin-1 and UTF-8 contain the ASCII charset, so checks with file <file to check> may still return the ASCII charset, if the file doesn't contain any non-ASCII chars
		#ifdef Q_OS_WIN32
		stream->setCodec("ISO 8859-1");
		#else
		stream->setCodec("UTF-8");
		//stream->setGenerateByteOrderMark(true); //NOT RECOMMENDED! javac *.java on such files will even generate an error!
		#endif
	}
	else stream->setCodec(m_codec.toLocal8Bit().constData()); //NOTE: if the codec isn't available, nothing happens
}

QString TextFileReadWriteHelper::readFile(const QString& path, bool* ok)
{
if (ok != 0) *ok = false;

QFile file(path);
if (!file.open(QIODevice::ReadOnly | QIODevice::Text)) return ""; //ok is still false...
QTextStream in(&file);
setQTextStreamCodec(&in);
QString ret = in.readAll();
file.close();

if (ok != 0) *ok = true;
return ret;
}

void TextFileReadWriteHelper::writeFile(const QString& path, const QString& content, bool* ok)
{
if (ok != 0) *ok = false;

QFile file(path);
if (!file.open(QIODevice::WriteOnly | QIODevice::Text)) return;
QTextStream out(&file);
setQTextStreamCodec(&out);
out << content;
file.close();

if (ok != 0) *ok = true;
}

QStringList& TextFileReadWriteHelper::readFileToList(const QString& path, QStringList& out, bool* ok)
{
if (ok != 0) *ok = false;

QFile file(path);
if (!file.open(QIODevice::ReadOnly | QIODevice::Text)) return out;
QTextStream in(&file);
setQTextStreamCodec(&in);
while (!in.atEnd()) out.append(in.readLine());
file.close();

if (ok != 0) *ok = true;
return out;
}

void TextFileReadWriteHelper::writeFileFromList(const QString& path, const QStringList& content, bool* ok)
{
if (ok != 0) *ok = false;

QFile file(path);
if (!file.open(QIODevice::WriteOnly | QIODevice::Text)) return;
QTextStream out(&file);
setQTextStreamCodec(&out);
for (int i = 0; i < content.size(); ++i) out << content[i] << "\n";
file.close();

if (ok != 0) *ok = true;
}
