/*-
* renameFiles is a simple tool to rename many files at once
* Copyright (C) 2011 David Hobach
* tripleh [at] hackingthe [dot] net
* 
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
-*/

/** simple class to help read and write text files in a platform independent way
 * if the desired codec isn't available, nothing happens
 * unfortunately Qt doesn't seem to guess the codecs correctly, so we have to do it
 * @since Qt 4.6
 * @author David Hobach
 * @version 0.3 / 12.04.2011
 */

#ifndef TEXTFILEREADWRITEHELPER_H
#define TEXTFILEREADWRITEHELPER_H

#include <QStringList>
#include <QString>
#include <QTextStream>

class TextFileReadWriteHelper {

public:

/** ctor
 * @param codec to use for reading and writing files (optional); if not specified, a best fit is guessed
 */
TextFileReadWriteHelper(const QString& codec = "") {m_codec = codec;}

//getters and setters
void setCodec(const QString& codec) {m_codec = codec;}
QString getCodec() {return m_codec;}

/** read a text file
 * @param path to the file
 * @param output parameter: set to true, if everything went ok; set to false otherwise (optional)
 * @return content of the file
 */
QString readFile(const QString& path, bool* ok = 0);

/** write a text file
 * @param path to the file
 * @param content to write
 * @param output parameter: set to true, if everything went ok; set to false otherwise (optional)
 */
void writeFile(const QString& path, const QString& content, bool* ok = 0);

/** read a text file and append its content to the given list, line by line
 * @param path to the file
 * @param list to write to (output parameter)
 * @param output parameter: set to true, if everything went ok; set to false otherwise (optional)
 * @return content of the file appended to the given list: each list item representing one line in the real file (newline characters removed)
 */
QStringList& readFileToList(const QString& path, QStringList& out, bool* ok = 0);

/** write a text file
 * @param path to the file
 * @param content to write; each item representing one line in the output file
 * @param output parameter: set to true, if everything went ok; set to false otherwise (optional)
 */
void writeFileFromList(const QString& path, const QStringList& content, bool* ok = 0);

private:
QString m_codec;

//set m_codec for the stream or guess an appropriate codec
void setQTextStreamCodec(QTextStream* stream);

}; //end class

#endif
