/*-
* renameFiles is a simple tool to rename many files at once
* Copyright (C) 2011 David Hobach
* tripleh [at] hackingthe [dot] net
* 
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
-*/

/** QObject Wrapper classes, which enable us to use basic types in the same way as QObjects
* every wrapper has a property containing the basic type data
* NOTE: QObject enforces private copy constructors!!
* implementation notes:
* templates couldn't be used, because QObject doesn't work with templates:
* http://lists.trolltech.com/qt-interest/2006-02/thread00693-0.html
* http://doc.trolltech.com/qq/qq15-academic.html
* QVariant couldn't be used, because it uses templates...
* @author David Hobach
* @version 0.3 / 16.04.2010
*/

#ifndef WRAPPERS_H
#define WRAPPERS_H

#include <QObject>
#include <QString>
#include <QByteArray>
#include <QChar>

//TODO s. txt

namespace wrappers
{

/*************************************************** WInteger *****************************************************/

class WInteger : public QObject
{
Q_OBJECT

Q_PROPERTY (int value READ getValue WRITE setValue)

public:

WInteger(const int value) : m_val(value) {;}

~WInteger() {;}

int getValue() const {return m_val;}

void setValue(const int value) {m_val = value;}

//conversion operator:
operator int() {return m_val;}

private:
int m_val;

}; //end class

/*************************************************** WFloat *****************************************************/

class WFloat : public QObject
{
Q_OBJECT

Q_PROPERTY (float value READ getValue WRITE setValue)

public:

WFloat(const float value) : m_val(value) {;}

~WFloat() {;}

float getValue() const {return m_val;}

void setValue(const float value) {m_val = value;}

//conversion operator:
operator float() {return m_val;}

private:
float m_val;

}; //end class

/*************************************************** WDouble *****************************************************/

class WDouble : public QObject
{
Q_OBJECT

Q_PROPERTY (double value READ getValue WRITE setValue)

public:

WDouble(const double value) : m_val(value) {;}

~WDouble() {;}

double getValue() const {return m_val;}

void setValue(const double value) {m_val = value;}

//conversion operator:
operator double() {return m_val;}

private:
double m_val;

}; //end class

/*************************************************** WString *****************************************************/

class WString : public QObject
{
Q_OBJECT

Q_PROPERTY (QString value READ getValue WRITE setValue)

public:

WString(const QString& value) : m_val(value) {;}

~WString() {;}

QString& getValue() {return m_val;} //without const in this case

void setValue(const QString& value) {m_val = value;}

//conversion operator:
operator QString&() {return m_val;}

private:
QString m_val;

}; //end class

/*************************************************** WByteArray *****************************************************/

class WByteArray : public QObject
{
Q_OBJECT

Q_PROPERTY (QByteArray value READ getValue WRITE setValue)

public:

WByteArray(const QByteArray& value) : m_val(value) {;}

~WByteArray() {;}

QByteArray& getValue() {return m_val;} //without const in this case

void setValue(const QByteArray& value) {m_val = value;}

//conversion operator:
operator QByteArray&() {return m_val;}

private:
QByteArray m_val;

}; //end class

/*************************************************** WChar *****************************************************/

class WChar : public QObject
{
Q_OBJECT

Q_PROPERTY (QChar value READ getValue WRITE setValue)

public:

WChar(const QChar& value) : m_val(value) {;}

~WChar() {;}

QChar& getValue() {return m_val;} //without const in this case

void setValue(const QChar& value) {m_val = value;}

//conversion operator:
operator QChar&() {return m_val;}

private:
QChar m_val;

}; //end class

} //end namespace

#endif
