/*-
* renameFiles is a simple tool to rename many files at once
* Copyright (C) 2011 David Hobach
* tripleh [at] hackingthe [dot] net
* 
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
-*/

#include <QXmlInputSource>
#include <QXmlSimpleReader>
#include <QString>
#include <QVariant>
#include <QDebug>

#include "XMLQObjectReader.h"
#include "QObjectFactory.h"

using namespace factories;

XMLQObjectReader::XMLQObjectReader(QObjectFactory* factory) : m_Factory(factory)
{
Q_ASSERT(factory != 0);
}

QList<QObject*> XMLQObjectReader::fromBytes(const QByteArray& arr, QObject* parent)
{
QString input(arr);
QXmlInputSource source;
source.setData(input);
QXmlSimpleReader reader;

//init members
m_Current = 0;
m_ObjectList.clear();
m_ParentStack.clear();

reader.setContentHandler(this);
	if (!reader.parse(source)) //parse() returns true, if successful
	{
	qDebug() << "ERROR: READER RETURNED FALSE!";
	return QList<QObject*>();
	}
return m_ObjectList; //root elements (might be empty, if none were parsed)
}

bool XMLQObjectReader::startElement(const QString &, const QString & elementName, const QString &, const QXmlAttributes & atts)
{
//Unnamed parameters are a way of avoiding "parameter not used" warnings from the compiler. It is necessary to include the parameters, even though we do not need them for this application, so that the signature matches that of the base class method and polymorphic overrides will be properly called.
	if (elementName == "object")
	{
		if (m_Current != 0) m_ParentStack.push(m_Current); //If we are already inside an <object> -->Keep track  of the current parent
		QString classname = atts.value("class");
		QString instancename = atts.value("name");
		m_Current = m_Factory->build(classname); //sets the parent to the factory
			if (!m_Current) //factory may return a null pointer --> error
			{
			qDebug() << "Error: Factory returned NULL!";
			return false;
			}
		m_Current->setObjectName(instancename);
		
		if (!m_ParentStack.empty()) //If this element has a parent, it is on the top of the stack. Set its parent.
		{
		m_Current->setParent(m_ParentStack.top());
		}
		return true;
	}
	
	if (elementName == "property") 
	{
	QString fieldType = atts.value("type");
	QString fieldName = atts.value("name");
	QString fieldValue = atts.value("value");
	QVariant qv = QVariant(fieldValue); //note that the type doesn't seem relevant for static properties as the class description already contains the type; however, for dynamic properties, this should be different! --> explicitly set the type
		if (!qv.convert(QVariant::nameToType(fieldType.toLocal8Bit())))
		{
		qDebug() << "ERROR: Couldn't set type of property: " << fieldName.toLocal8Bit();
		return false; //error
		}
	m_Current->setProperty(fieldName.toLocal8Bit(),qv); //setProperty returns false for dynamic properties!
	}
return true;
}

bool XMLQObjectReader::endElement(const QString&, const QString& elementName, const QString&)
{
	if (elementName == "object")
	{
	if (!m_ParentStack.empty()) {m_Current = m_ParentStack.pop();}
	else addCurrentToList(); //m_current is a root object
	}
return true;
}

bool XMLQObjectReader::endDocument()
{
return true;
}

void XMLQObjectReader::addCurrentToList()
{
	if (m_Current != 0) 
	{
	m_ObjectList.append(m_Current);
	m_Current = 0;
	}
}
