/*-
* renameFiles is a simple tool to rename many files at once
* Copyright (C) 2011 David Hobach
* tripleh [at] hackingthe [dot] net
* 
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
-*/

/** implementation of the interface AbstractQObjectReader for XML input
* NOTE: add QT += xml to your qt project file
* corresponding writer: XMLQObjectWriter
* @author David Hobach
* @version 0.3 / 12.04.2010
*/

#ifndef XMLQOBJECTREADER_H
#define XMLQOBJECTREADER_H


#include <QXmlDefaultHandler>
#include <QStack>

#include "AbstractQObjectReader.h"
#include "QObjectFactory.h"

using namespace factories;

class XMLQObjectReader : public AbstractQObjectReader, protected QXmlDefaultHandler
{

public:

/** ctor
* @param QObjectFactory to use for the creation of all QObjects (optional); default: singleton instance of QObjectFactory (but make sure to register the needed Constructors first!!!!)
*/
XMLQObjectReader(QObjectFactory* factory = QObjectFactory::instance());

/** read a QByteArray containing XML content and construct a QObject from it
* @param QByteArray to read from
* @param parent QObject for the QObject that was read (optional); if set to NULL, the parent will be the factory used; hence the factory should be destroyed correctly!!!
* @return list of pointers to the constructed root QObjects (may have multiple subobjects) or an empty QList(), if the operation failed
*/
virtual QList<QObject*> fromBytes(const QByteArray& arr, QObject* parent = 0);

/** dtor
*/
virtual ~XMLQObjectReader() {;}

//inherited from AbstractQObjectReader:
//virtual QList<QObject*> readFromFile(const QString& path, QObject* parent = 0);

protected:
// callback methods from QXmlDefaultHandler
bool startElement(const QString& namespaceURI, const QString& name, const QString& qualifiedName, const QXmlAttributes& attributes); //The reader calls this function when it has parsed a start element tag.
bool endElement(const QString& namespaceURI, const QString& localName, const QString& qualifiedName); //The reader calls this function when it has parsed an end element tag with the qualified name qName, the local name localName and the namespace URI namespaceURI. If this function returns false the reader stops parsing and reports an error.
bool endDocument(); //The reader calls this function after it has finished parsing. It is called just once, and is the last handler function called. It is called after the reader has read all input or has abandoned parsing because of a fatal error.

private:
QObjectFactory* m_Factory;
QObject* m_Current; //pointer to the QObject, which was created last for usage as a potential parent
QList<QObject*> m_ObjectList; //contains all root objects created during parsing
QStack<QObject*> m_ParentStack; //top element is the current parent

/** add m_current as a root QObject to the m_ObjectList and set m_Current to 0
*/
void addCurrentToList();

}; //end class

#endif
