/*-
* renameFiles is a simple tool to rename many files at once
* Copyright (C) 2011 David Hobach
* tripleh [at] hackingthe [dot] net
* 
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
-*/

#include <QMetaObject>
#include <QMetaProperty>

#include "XMLQObjectWriter.h"

XMLQObjectWriter::XMLQObjectWriter(QVariantHelper* qvh)
{
	if (qvh == 0)
	{
	m_allocated = true;
	m_qvh = new QVariantHelper();
	}
	else
	{
	m_allocated = false;
	m_qvh = qvh;
	}
}

QByteArray XMLQObjectWriter::toBytes(const QObject* obj, bool writeChildren)
{
QString ret = this->toString(obj,writeChildren);
if (ret == QString()) return QByteArray();
return ret.toLocal8Bit();
}

QString XMLQObjectWriter::toString(const QObject* obj, bool writeChildren)
{
QStringList ret;
if (!this->toString(ret,obj,writeChildren,0)) return QString();
return ret.join("\n");
}

bool XMLQObjectWriter::toString(QStringList& result, const QObject* obj, bool writeChildren, int indentLevel) const
{
QString indentspace, value;
indentspace.fill('\t', indentLevel);
QString className = obj->metaObject()->className();
QString objectName = obj->objectName();

result << QString("%1<object class=\"%2\" name=\"%3\">").arg(indentspace).arg(className).arg(objectName);

QStringList propnames = this->propertyNames(obj); //inherited
	foreach (QString propName, propnames)
	{
	if (propName == "objectName") continue; //already done
	QVariant qv = obj->property(propName.toLocal8Bit());

		if (propName == "className") 
		{
		className = qv.toString();
		continue;
		}
	
	const QMetaObject* meta = obj->metaObject();
	int idx = meta->indexOfProperty(propName.toLocal8Bit());
	QMetaProperty mprop = meta->property(idx);
	
	bool ok;
	//NOTE: newlines aren't a problem (except for readability maybe), because they are perfectly allowed in XML
	value = m_qvh->toString(qv,mprop,ok).toHtmlEscaped(); //otherwise XML parsers might get problems with non-ASCII characters //note: QString() is returned by toString(), if the QVariant can't be converted
	if (!ok) return false;
	result << QString("%1\t<property name=\"%2\" type=\"%3\" value=\"%4\"/>").arg(indentspace).arg(propName).arg(qv.typeName()).arg(value);
	}
	
	if (writeChildren) //work recursively, if flag set
	{
	QList<QObject*> childlist = obj->findChildren<QObject*>(QString()); //equivalent to QList<QObject*> childlist = obj->findChildren<QObject*>(QString());

		foreach (QObject* child, childlist)
		{
			if (child->parent() != obj)
			{
			//qDebug() << "This is not my child!!";
			continue;
			}
			if ((child != 0) && (!this->toString(result,child,writeChildren,indentLevel+1))) return false;
		}
	}

result << QString("%1</object>").arg(indentspace); //for some people, "%1</object>\n" might be more readable...
return true; //if we get here, the conversion QVariant --> QString of the value was possible at least in this run of the recursion
}

XMLQObjectWriter::~XMLQObjectWriter()
{
	if ((m_allocated) && (m_qvh != 0))
	{
	delete m_qvh;
	m_qvh = 0;
	}
}
