/*-
* renameFiles is a simple tool to rename many files at once
* Copyright (C) 2011 David Hobach
* tripleh [at] hackingthe [dot] net
* 
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
-*/

/** implementation of the interface AbstractQObjectWriter for XML output
* corresponding reader: XMLQObjectReader
* @author David Hobach
* @version 0.3 / 12.04.2010
*/

#ifndef XMLQOBJECTWRITER_H
#define XMLQOBJECTWRITER_H

#include "AbstractQObjectWriter.h"
#include "QVariantHelper.h"

class XMLQObjectWriter : public AbstractQObjectWriter
{

public:

/** ctor
* @param pointer to a QVariantHelper to use for handling the various types that may occur in the QObjects to write with this writer (if set to NULL, the basic QVariantHelper implementation is used)
*/
XMLQObjectWriter(QVariantHelper* qvh = 0);

/** get a QByteArray XML representation of the specified QObject
* @param pointer to QObject to represent as QByteArray in XML
* @param whether or not to inspect and recursively write child objects of the object to write
* @return QByteArray XML representation of the specified QObject; it has size 0, if the operation wasn't successful; NOTE: since the return value is XML, the function just calls this.toString().toAscii()
*/
virtual QByteArray toBytes(const QObject* obj, bool writeChildren = true);

/** same as toBytes, but returns the XML representation of the QObject as QString
* @param pointer to the QObject to represent in XML
* @param whether or not to inspect and recursively write child objects of the object to write (optional)
* @return XML QString representation of the QObject; == QString(), if the operation wasn't successful (if some properties had types, that couldn't be converted from QVariant)
*/
virtual QString toString(const QObject* obj, bool writeChildren = true);

//implementation of base class is sufficient
//bool writeToFile(const QString& path, const QObject* obj, bool writeChildren = true);

/** dtor
*/
virtual ~XMLQObjectWriter();


protected:

/** same as the public version, just with specification of the indentlevel for the implementation of recursion
* @param QStringList to use for the return value: XML QString representation of the QObject and possibly all of its children with exactly one non-empty QString per line
* @param pointer to the QObject to represent in XML
* @param whether or not to inspect and recursively write child objects of the object to write (optional)
* @param level of indention for recursion (uses tabs)
* @return whether all values of the properties could be correctly converted by the QVariantHelper = true
*/
virtual bool toString(QStringList& result, const QObject* obj, bool writeChildren, int indentLevel) const;

private:
bool m_allocated; //whether m_qvh was allocated by this class or not
QVariantHelper* m_qvh;

}; //end class

#endif
